# -*- coding: utf-8 -*-
import numpy as np

class ConvertUtility:
  """
  An utility class used for converting image to the nord palette

  Methods
  -------
  color_difference(color1, color2)
    Find the color difference between the two given colors

  get_avg_color(pixels, row, col, w, h)
    Get the avg color of a given area and return it as tuple containing rgb
  """

  def color_difference(color1, color2):
    """
    Find the color difference between the two given colors

    Parameters
    ----------
    color1 : tuple
        color in rgb
    color2 : tuple
        color in rgb

    Returns
    -------
    tuple
      the resultant color
    """
    return sum([abs(component1-component2) for component1, component2 in zip(color1, color2)])

  def get_avg_color(pixels, row, col, w=-2, h=3):
    """
    Get the avg color of a given area and return it as tuple containing rgb

    Parameters
    ----------
    pixels : dict
      The pixel map of the source image
    row : int
      Row counter where to start
    col : int
      Col counter where to start
    w : int
      Box's wdith
    h : int
      Box's height

    Returns
    -------
    tuple
      the resultant color in rgb format
    """
    average_sum = []
    for k in range(w, h):
      for l in range(w, h):
        try:
          average_sum.append(pixels[row+k, col+l])
        except:
          pass

    size = len(average_sum)
    if (size <= 0):
      size = 1

    r = 0
    g = 0
    b = 0
    a = 255
    for x in average_sum:
      r += x[0]
      g += x[1]
      b += x[2]
      if (len(x) > 3):
        a += x[3]

    avg_color = (int(r/size), int(g/size), int(b/size))
    if (a != 255):
      avg_color = avg_color + (int(a/size), )

    return avg_color
  
  def convert_palette(color_cube, image):
    """Convert frame color palette

    Parameters
    ----------
    color_cube: ndarray
      Color map of RGB colorspace created from palette colors
    image: ndarray
      Current frame

    Returns
    -------
    ndarray
      color converted frame
    """

    shape = image.shape[0:2]
    indices = image.reshape(-1,3)
    # Pass image colors and retrieve corresponding palette color
    new_image = color_cube[indices[:,0],indices[:,1],indices[:,2]]

    return new_image.reshape(shape[0],shape[1],3).astype(np.uint8)




"""This is the example module.

This module does stuff.
"""
from os import listdir
import numpy as np


def load_palette_set(path):
    """Create a list of every colors set on the path given.

      <Description>

    Parameters
    ----------
    <argument name>: <type>
      <argument description>
    <argument>: <type>
      <argument description>

    Returns
    -------
    <type>
      <description>
    """
    directories = listdir(path)

    palette_list = [palette_file.replace(
        ".txt", '') for palette_file in directories]

    return palette_list


def find_palettes(path):
    """Create a set with every palettes stored in the directory given.

      <Description>

    Parameters
    ----------
    <argument name>: <type>
      <argument description>
    <argument>: <type>
      <argument description>

    Returns
    -------
    <type>
      <description>
    """
    palettes = [palette.lower() for palette in listdir(path)]
    return palettes


def import_palette_from_file(filename):
    """<Short Description>

      <Description>

    Parameters
    ----------
    <argument name>: <type>
      <argument description>
    <argument>: <type>
      <argument description>

    Returns
    -------
    <type>
      <description>
    """
    opened_file = open(filename, "r")
    palette = [line.replace('#', '').replace('\n', '')
               for line in opened_file.readlines()]
    return palette


def create_data_colors(palette):
    """<Short Description>

      <Description>

    Parameters
    ----------
    <argument name>: <type>
      <argument description>
    <argument>: <type>
      <argument description>

    Returns
    -------
    <type>
      <description>
    """
    data = []
    for color in palette:
        data.extend((export_tripletes_from_color(color)))
    return data


def export_tripletes_from_color(hex_color):
    """<Short Description>

      <Description>

    Parameters
    ----------
    <argument name>: <type>
      <argument description>
    <argument>: <type>
      <argument description>

    Returns
    -------
    <type>
      <description>
    """
    hex_triplets = [hex_color[i:i+2] for i in range(0, len(hex_color), 2)]
    triplets_integer = [int(hex_triplets[i], 16)
                        for i in range(len(hex_triplets))]
    return triplets_integer


def generate_color_map(palette, palette_name):
  """ Generate a color map

    Generate a color map of entire RGB color space adapted to the color palette
    The function maps every color to the closest color in the palette before conversion process
    This increases speed of color conversion as there is only lookups during eecution time

  Parameters
  ----------
  palette: ndarray / list
    Contains the palette in ndarray form(RGB values split)
  palette_name: string
    Name of the color palette

  Returns
  -------
  None
    Generates a .npz file and saves it to disk
  """
  if not isinstance(palette, np.ndarray):
    palette = np.asarray(palette)
  precalculated = np.zeros(shape=[256,256,256,3])
  for i in range(256):
    for j in range(256):
      for k in range(256):
        index = np.argmin(np.sqrt(np.sum(
            ((palette)-np.array([i,j,k]))**2,
            axis=1
          )))
        precalculated[i,j,k] = palette[index]
  np.savez_compressed(palette_name, color_cube = precalculated)    


import pathlib
from setuptools import setup, find_packages

ROOT = pathlib.Path('.')
README = (ROOT / "README.md").read_text()

setup(
    name="image-go-nord",
    version="1.2.0",
    description="A tool to convert any RGB image or video to any theme or color palette input by the user",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/Schrodinger-Hat/ImageGoNord-pip",
    download_url = 'https://github.com/Schrodinger-Hat/ImageGoNord-pip/releases',
    keywords = ['nordtheme', 'pillow', 'image', 'conversion', 'rgb', 'color-scheme', 'color-palette', 'linux-rice', 'gruvbox', 'catpuccin'], 
    author="Schrodinger Hat",
    author_email="schrodinger.hat.show@gmail.com",
    license="AGPL-3.0",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        "License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7"
    ],
    project_urls={
        "Homepage": "https://ign.schrodinger-hat.it",
        "Source": "https://github.com/Schrodinger-Hat/ImageGoNord-pip",
        "Bug Reports": "https://github.com/Schrodinger-Hat/ImageGoNord-pip/issues",
    },
    packages=find_packages(),
    package_data={'': ['*.txt', 'palettes/*.txt']},
    include_package_data=True,
    install_requires=["Pillow", "ffmpeg-python", "numpy", "requests"],
    extras_require = {
        'AI':  ["torch", "scikit-image", "torchvision"]
    },
    python_requires=">=3.5"
)
